package org.geekfu.MapPanel;

import info.clearthought.layout.TableLayout;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.geekfu.OldCartographer.Map;

public class MapPanel extends JPanel implements ActionListener{

	protected Map map;
	protected JScrollPane mapscroll;
	protected JPanel sidebar;

	protected FireableTextField textentry;
	protected JTextArea textoutput;
	protected JScrollPane textscroll;
	
	protected JFrame textWindow;
	protected JTextArea windowTextEntry;
	
	protected List<MapPanelListener> listeners=new LinkedList<MapPanelListener>();

	private boolean text;

	public static final int BORDER=10;
	private static final String IMAGEPATH = "/org/geekfu/hippo/blueHippo.png";

	/**
	 * @param arg0
	 * @param arg1
	 * @param map
	 * @param sidebar
	 * @param text
	 */
	public MapPanel(Map map, JPanel sidebar, boolean text) {
		super();

		init(map,sidebar,text);
	}

	protected MapPanel() {}

	protected void init(Map map) {
		init(map,null,false);
	}

	protected void init(Map map, JPanel sidebar, boolean text) {
		this.map = map;
		this.sidebar = sidebar;
		this.text=text;

		double[][] sizes= {
				{BORDER, TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, BORDER},
				{
					BORDER, 
					TableLayout.FILL, 
					0.25, 
					TableLayout.PREFERRED, 
					BORDER},
		};

		setLayout(new TableLayout(sizes));

		setMap(map);

		if(sidebar!=null) {
			add(sidebar,"3, 1, 3, 3");
		}

		textentry=new FireableTextField();
		textoutput=new JTextArea();
		textoutput.setEditable(false);
		textscroll = new JScrollPane(textoutput);
		textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		JButton showTextWindow=new JButton("...");
		showTextWindow.addActionListener(this);
		
		add(textscroll,"1, 2, 2, 2");
		add(textentry,"1, 3");
		add(showTextWindow,"2, 3");
		if(!text)
		{
			((TableLayout) getLayout()).setRow(2,0);
			((TableLayout) getLayout()).setRow(3,0);
		}

		validate();
	}

	public void scrollToCell(int x,int y) {
		if(map==null)return;
		Dimension c=map.getCellsize();
		Rectangle r=new Rectangle(x*c.width,y*c.height,c.width,c.height);
		map.scrollRectToVisible(r);
	}


	public void addActionListener(MapPanelListener a) {
		listeners.add(a);
	}

	public void showString(String message){
		if(!text)return;
		textoutput.append(message);
		if(!message.endsWith("\n"))
			textoutput.append("\n");
		textoutput.scrollRectToVisible(new Rectangle(0,textoutput.getHeight(),1,1));
	}
	
	public Map getMap() {
		return map;
	}

	public JPanel getSidebar() {
		return sidebar;
	}

	public boolean isText() {
		return text;
	}

	public void setMap(Map map) {
		this.map = map;
		if(mapscroll!=null)
			remove(mapscroll);

		if(map!=null){
			mapscroll=new JScrollPane(new CenteredPanel(map));
			mapscroll.getVerticalScrollBar().setUnitIncrement(map.getCellsize().height/2);
			mapscroll.getHorizontalScrollBar().setUnitIncrement(map.getCellsize().width/2);
		}else{
			try {
				Image img=ImageIO.read(getClass().getResource(IMAGEPATH));
				JComponent imgpanel=new ImageComponent(img);
				mapscroll=new JScrollPane(imgpanel);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		add(mapscroll,"1, 1, 2, 1");
		validate();
	}

	public void setSidebar(JPanel sidebar) {
		this.sidebar = sidebar;
	}

	public void setText(boolean text) {
		this.text = text;
		((TableLayout) getLayout()).setRow(2,(isText()?TableLayout.PREFERRED:0));
		((TableLayout) getLayout()).setRow(3,(isText()?TableLayout.PREFERRED:0));
		validate();
	}

	private class ImageComponent extends JComponent{
		private Image img;
		public ImageComponent(Image img){
			this.img=img;
		}
		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			int w=this.getWidth();
			int h=this.getHeight();
			int iw=img.getWidth(null);
			int ih=img.getHeight(null);
			g.drawImage(img, w/2-iw/2, h/2-ih/2, null);
		}
		@Override
		public Dimension getPreferredSize() {
			return new Dimension(img.getWidth(null),img.getHeight(null));
		}
	}

	private class CenteredPanel extends Container implements LayoutManager{
		public JComponent comp;
		public CenteredPanel(JComponent comp){
			this.comp=comp;
			add(comp);
			setLayout(this);
		}
		@Override
		public Dimension getPreferredSize() {
			return new Dimension(comp.getWidth(),comp.getHeight());
		}
		public void layoutContainer(Container arg0) {
			int basex=0,basey=0;
			Dimension d=comp.getPreferredSize();
			int reqx=d.width;
			int reqy=d.height;
			if(getWidth()>reqx) //wider than we need
			basex=getWidth()/2-reqx/2;
			if(getHeight()>reqy) //taller than we need
			basey=getHeight()/2-reqy/2;
			comp.setBounds(basex,basey,d.width,d.height);
		}
		public Dimension minimumLayoutSize(Container arg0) {
			return getPreferredSize();
		}
		public Dimension preferredLayoutSize(Container arg0) {
			return getPreferredSize();
		}
		public void addLayoutComponent(String arg0, Component arg1) {}
		public void removeLayoutComponent(Component arg0) {}
	}

	public void actionPerformed(ActionEvent arg0) {
		if(textWindow==null){
			textWindow=new JFrame();
			textWindow.setSize(300,400);
			windowTextEntry=new JTextArea();
			
			JScrollPane textscroll = new JScrollPane(windowTextEntry);
			textscroll.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

			textWindow.setLayout(new BorderLayout());
			
			textWindow.getContentPane().add(textscroll,BorderLayout.CENTER);
			JButton enter=new JButton("Enter");
			enter.addActionListener(new WindowButtonListener());
			textWindow.getContentPane().add(enter,BorderLayout.SOUTH);
		}
		
		windowTextEntry.setText(textentry.getText());
		textWindow.setVisible(true);
	}

	private class WindowButtonListener implements ActionListener{
		public void actionPerformed(ActionEvent arg0) {
			String str=windowTextEntry.getText();
			textWindow.setVisible(false);
			textentry.setText(str);
			textentry.fireAction();
		}
	}
	
	private class FireableTextField extends JTextField implements KeyListener,ActionListener{
		private LinkedList<String> values=new LinkedList<String>();
		private int hist=-1;
		private String working="";
		public FireableTextField(){
			addKeyListener(this);
			addActionListener(this);
		}
		public void fireAction(){
			fireActionPerformed();
		}

		public void keyReleased(KeyEvent event) {}
		public void keyTyped(KeyEvent event) {}
		public void keyPressed(KeyEvent event) {
			if(event.getKeyCode()==KeyEvent.VK_UP){
				if(hist==-1){
					working=getText();
				}else{
					values.set(hist, getText());
				}
				if(hist<values.size()-1){
					hist++;
					setText(values.get(hist));
				}
			}else if(event.getKeyCode()==KeyEvent.VK_DOWN){
				if(hist==-1){
					working=getText();
				}
				else{
					values.set(hist, getText());
				}
				
				if(hist>0){
					hist--;
					setText(values.get(hist));
				}else if(hist>-1){
					hist--;
					setText(working);
				}
			}
		}
		public void actionPerformed(ActionEvent evt) {
			String str=getText();
			values.addFirst(str);
			hist=-1;
			working="";
			setText(working);
			for(MapPanelListener m : listeners){
				m.textEntered(str);
			}
		}
	}

	public JTextField getTextEntry() {
		return textentry;
	}
}