package org.geekfu.hippo;

import org.geekfu.OldCartographer.ColorCell;
import org.geekfu.OldCartographer.MapCell;
import org.mozilla.javascript.EvaluatorException;
import org.mozilla.javascript.ScriptableObject;

public class HostPiece extends ScriptableObject{

	private MapCell cell;
	private HostMap map;
	private String color;
	private int tilenum;
	private int x,y;
	
	private Object onclick;
	
	public String getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = color;
		((ColorCell)cell).setForeground(Utils.getColor(color));
		if(map!=null)map.repaint();
	}

	public int getTilenum() {
		return tilenum;
	}

	public void setTilenum(int tilenum) {
		this.tilenum = tilenum;
	}

	@Override
	public String getClassName() {
		return "Piece";
	}

	public void jsConstructor(Object param){
		defineProperty("map", this.getClass(), ScriptableObject.READONLY);
		defineProperty("x", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("y", this.getClass(), ScriptableObject.PERMANENT);
		defineProperty("onclick", this.getClass(), ScriptableObject.PERMANENT);
		defineFunctionProperties(new String[]{"move"}, this.getClass(), ScriptableObject.READONLY);
		if(param instanceof String){
			String color=param.toString();
			defineProperty("color", this.getClass(), ScriptableObject.PERMANENT);
			cell=new ColorCell(null,Utils.getColor(color),true);
			this.color=color;
		}
	}

	public MapCell getCell() {
		return cell;
	}

	public HostMap getMap() {
		return map;
	}

	public void setMap(HostMap newmap) {
		if(newmap!=map && map!=null && newmap!=null)
			map.remove(this);
		
		this.map = newmap;
	}
	
	public String toString(){
		return super.toString()+"("+color+")";
	}
	
	public Object getX(){
		if(map==null)return null;
		return x;
	}
	
	public Object getY(){
		if(map==null)return null;
		return y;
	}
	
	public void setX(int x){
		this.x=x;
		if(map!=null)map.place(this, x, y);
	}
	
	public void setY(int y){
		this.y=y;
		if(map!=null)map.place(this, x, y);
	}
	
	public void setPointNoNotify(int x,int y){
		this.x=x;this.y=y;
	}
	
	public void move(int endx,int endy,int duration,int steps){
		if(map==null)
			throw new EvaluatorException("This piece is not on a map");
		map.move(this, endx, endy, duration, steps);
	}
	
	
	public Object getOnclick() {
		return onclick;
	}

	public void setOnclick(Object onclick) {
		this.onclick = onclick;
	}
}
